// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.

#ifndef SST_CORE_SERIALIZATION_CORE_H
#define SST_CORE_SERIALIZATION_CORE_H

#if defined(SST_CORE_SERIALIZATION_ELEMENT_H) || defined(SST_CORE_SERIALIZATION_ARCHIVE_H)
#error "Include only one serialization/ header file"
#endif

#if SST_WANT_POLYMORPHIC_ARCHIVE
#include <boost/archive/polymorphic_iarchive.hpp>
#include <boost/archive/polymorphic_oarchive.hpp>
#else
#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#endif

#include <boost/serialization/base_object.hpp>
#include <boost/serialization/export.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/mpi.hpp>

#include "sst/core/serialization/types.h"

#if SST_WANT_POLYMORPHIC_ARCHIVE
#define SST_BOOST_SERIALIZATION_INSTANTIATE(func)                       \
    template void                                                       \
    func<boost::archive::polymorphic_iarchive>(                         \
                                    boost::archive::polymorphic_iarchive & ar, \
                                    const unsigned int file_version);   \
    template void                                                       \
    func<boost::archive::polymorphic_oarchive>(                         \
                                    boost::archive::polymorphic_oarchive & ar, \
                                    const unsigned int file_version);
#else
#define SST_BOOST_SERIALIZATION_INSTANTIATE(func)                       \
    template void                                                       \
    func<boost::archive::binary_iarchive>(                              \
                                    boost::archive::binary_iarchive & ar, \
                                    const unsigned int file_version);   \
    template void                                                       \
    func<boost::archive::binary_oarchive>(                              \
                                    boost::archive::binary_oarchive & ar, \
                                    const unsigned int file_version);
#endif

#endif // #ifndef SST_CORE_SERIALIZATION_CORE_H
