// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


#ifndef _PHXSIMC_H
#define _PHXSIMC_H

#include <sst/core/log.h>
#include <sst/core/event.h>
#include <sst/core/component.h>
#include <memoryChannel.h>
#include <Callback.h>
//#include <LinearChain.h> /* currently broken */
#include <SingleCube.h>
#include <PHXEvent.h>


/* THIS COMPLETELY CHANGES THE STREAM, COMMENT THIS LINE FOR REAL RUNS */
//#define TEST_MODE

using namespace std;
using namespace SST;

#ifndef PHXSIMC_DBG
#define PHXSIMC_DBG
#endif

class PHXSimC : public Component {

public: // functions

  PHXSimC( ComponentId_t id, Params_t& params );
  int Finish();

private: // types

  typedef MemoryChannel<uint64_t> memChan_t;

private: // functions

  PHXSimC( const PHXSimC& c );
  bool clock( Cycle_t  );

  inline PHXSim::PacketType convertType( memChan_t::event_t::reqType_t type );
  void packetReturn(PHXSim::Packet *pkt, uint64_t clockcycle);

  std::deque<PHXSim::Packet *> m_transQ;

  std::map <unsigned, PHXSim::Packet *> writeReturnMap;
  PHXSim::SingleCube *           m_memorySystem;
  memChan_t*              m_memChan;
  std::string             m_printStats;
  Link *advMem;
  int linksToUse;
  // unique transaction identifier
  unsigned int transaction_id;
  Log< PHXSIMC_DBG >&    m_dbg;
  Log<>&                  m_log;
  
  static const int FPMultLat = 4;
  static const int FPDivLat = 8;
  static const int FPAddLat = 3;
  bool hadFirstEvent;
  // counters
  uint64_t engFPMul;
  uint64_t engFPDiv;
  uint64_t engFPAdd;
  uint64_t engRW; /* should include addr calculation */
  uint64_t engPartialR;
  uint64_t engDidWork;
  uint64_t engFinishedUnit;

  // number of state machine engies
  int engines;
  void engRead(unsigned int addr, int eng, int size=64);
  void engWrite(unsigned int addr, int eng, int size=64);
  bool handleMatVec(PHXEvent*, int);
  bool handleForceCalc(PHXEvent*, int);
  bool handlePagerank(PHXEvent*, int);
  void handleAdvMem();
  void advanceEngine(int);
  void resetCounters();
  vector<int> engOutstandingReads;
  vector<int> engIdx;
  typedef map<unsigned int, int> transToEngMap_t;
  // map of read transaction IDs to the issuing Engine
  transToEngMap_t readTransToEng;
  transToEngMap_t writeTransToEng;
  vector<deque<PHXEvent*> > advMemEvents;

#if WANT_CHECKPOINT_SUPPORT
  BOOST_SERIALIZE {
    _AR_DBG(PHXSimC,"start\n");
    BOOST_VOID_CAST_REGISTER( PHXSimC*, Component* );
    ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP( Component );
    ar & BOOST_SERIALIZATION_NVP( bus );
    _AR_DBG(PHXSimC,"done\n");
  }
  SAVE_CONSTRUCT_DATA( PHXSimC ) {
    _AR_DBG(PHXSimC,"\n");
    ComponentId_t     id     = t->_id;
    Clock*            clock  = t->_clock;
    Params_t          params = t->params;
    ar << BOOST_SERIALIZATION_NVP( id );
    ar << BOOST_SERIALIZATION_NVP( clock );
    ar << BOOST_SERIALIZATION_NVP( params );
  } 
  LOAD_CONSTRUCT_DATA( PHXSimC ) {
    _AR_DBG(PHXSimC,"\n");
    ComponentId_t     id;
    Clock*            clock;
    Params_t          params;
    ar >> BOOST_SERIALIZATION_NVP( id );
    ar >> BOOST_SERIALIZATION_NVP( clock );
    ar >> BOOST_SERIALIZATION_NVP( params );
    ::new(t)PHXSimC( id, clock, params );
  } 
#endif
};

inline PHXSim::PacketType
PHXSimC::convertType( memChan_t::event_t::reqType_t type )
{
#ifndef TEST_MODE
  switch( type ) {
  case memChan_t::event_t::READ:
    return PHXSim::DATA_READ;
  case memChan_t::event_t::WRITE:
    return PHXSim::DATA_WRITE;
  case memChan_t::event_t::RMW:
  case memChan_t::event_t::INV_REQ:
  default:
    ;
  }
  m_dbg.write("WARNING: Returning Bogus packet type from request type %d\n", type);
  return (PHXSim::PacketType)-1; 
#else 
  //this is artificial just for the sake of testing returns; mostly writes, some reads
  static int number=1;
  PHXSim::PacketType p = (number++ % 64) == 0 ? PHXSim::DATA_READ : PHXSim::DATA_WRITE;
  m_dbg.write("==============WARNING: TEST_MODE (%d, type=%d)==============\n", number, p);
  return p; 
#endif
}

#endif 

