// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/* -*- C -*-
 *
 * Copyright 2008 Sandia Corporation. Under the terms
 * of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
 * Government retains certain rights in this software.
 * 
 * Copyright (c) 2008, Sandia Corporation
 * All rights reserved.
 * 
 * This file is part of the SST software package. For license
 * information, see the LICENSE file in the top level directory of the
 * distribution.
 */

#ifndef NETSIM_H
#define NETSIM_H

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 * completion queue entry
 */
struct netsim_cqe_t {
    uint32_t status;          /* error code; success = 0 */
    uint32_t data_len;        /* length of data delivered (to be
                                 delivered for probe */
    uint64_t send_match_bits; /* match bits from sender */
    uint64_t user_data;       /* from recv wqe */
};
typedef struct netsim_cqe_t netsim_cqe_t;

#define NETSIM_SUCCESS    0
#define NETSIM_ERROR      1
#define NETSIM_NOT_FOUND  2

/* Initialize netsim interface.  May be called multiple times. */
int netsim_init(void);

/* Finalize netsim interface.  May be called exactly once. */
int netsim_finalize(void);
  
/* number of processes in the parallel job.  may be called before netsim_init. */
int netsim_size(void);
  
/* my rank in the parallel job.  may be called before netsim_init. */
int netsim_rank(void);

/* start a send */
int netsim_tx_start(uint32_t dest, void *buf, int msgSize,
                    int synchronous, uint64_t match_bits,
                    uint64_t user_data);
  
/* start a recv */
int netsim_rx_start(void *buf, int msgSize, 
                    uint64_t match_bits, uint64_t ignore_bits, 
                    uint64_t user_data);

/* start a probe */
int netsim_probe_start(uint64_t match_bits, uint64_t ignore_bits, 
                       uint64_t user_data);

/* return a completion queue entry.  Returns -1 if empty */
int netsim_cq_poll(netsim_cqe_t *cqe);

/* get time (clockcycles) */ 
double netsim_get_clock(void);

#ifdef __cplusplus
}
#endif

#endif /* ifndef NETSIM_H */
