// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


#include <stdlib.h>

#include "mpi_internal.h"

int dtype_sizes[MPI_TYPE_MAX];
int mpi_comm_world_rank = 0;
int mpi_comm_world_size = 0;
int mpi_error_abort = 0;
pool<request> *request_pool = NULL;

static int mpi_init_called = 0;
static int mpi_finalize_called = 0;

int
MPI_Abort(MPI_Comm comm, int code)
{
    printf("MPI Rank %d called MPI_Abort with code %d\n", 
           mpi_comm_world_rank, code);
    abort();

    return MPI_SUCCESS;
}


int
MPI_Comm_rank(MPI_Comm comm, int *rank)
{
    *rank = mpi_get_rank(comm);
    return mpi_error_check("MPI_Comm_rank", (*rank < 0) ? MPI_ERR_COMM : MPI_SUCCESS);
}


int
MPI_Comm_size(MPI_Comm comm, int *size)
{
    *size = mpi_get_size(comm);
    return mpi_error_check("MPI_Comm_size", (*size < 0) ? MPI_ERR_COMM : MPI_SUCCESS);
}


int
MPI_Finalize(void)
{
    if (mpi_finalize_called != 0 || mpi_init_called == 0) {
        return mpi_error_check("MPI_Finalize", MPI_ERR_OTHER);
    }
    mpi_finalize_called = 1;

    if (NULL != request_pool) delete request_pool;

    return mpi_error_check("MPI_Finalize", netsim_finalize());
}


int
MPI_Finalized(int *flag)
{
    *flag = mpi_finalize_called;
    return MPI_SUCCESS;
}


int
MPI_Init(int *argc, char ***argv)
{
    if (mpi_init_called != 0) {
        return mpi_error_check("MPI_Init", MPI_ERR_OTHER);
    }

    mpi_init_called = 1;

    dtype_sizes[MPI_BYTE] = 1;
    dtype_sizes[MPI_CHAR] = sizeof(char);
    dtype_sizes[MPI_SHORT] = sizeof(short);
    dtype_sizes[MPI_INT] = sizeof(int);
    dtype_sizes[MPI_LONG] = sizeof(long);
    dtype_sizes[MPI_FLOAT] = sizeof(float);
    dtype_sizes[MPI_DOUBLE] = sizeof(double);

    request_pool = new pool<request>;

    mpi_error_abort = 1;

    mpi_comm_world_size = netsim_size();
    mpi_comm_world_rank = netsim_rank();

    return mpi_error_check("MPI_Init", netsim_init());
}


int
MPI_Initialized(int *flag)
{
    *flag = mpi_init_called;
    return MPI_SUCCESS;
}


int
MPI_Init_thread(int *argc, char ***argv, int requested, int *provided)
{
    if (requested != MPI_THREAD_MULTIPLE) {
        *provided = requested;
    } else {
        provided = MPI_THREAD_SINGLE;
    }

    return MPI_Init(argc, argv);
}


double
MPI_Wtick(void)
{
    /* BWB - FIX ME - what does this do? */
    return 0.0001;
}


double
MPI_Wtime(void)
{
    return netsim_get_clock();
}
