// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


#include "mpi_internal.h"


int
MPI_Irsend(void *buf, int count, MPI_Datatype dtype, 
           int dest, int tag, MPI_Comm comm, 
           MPI_Request *req)
{
    return mpi_error_check("MPI_Irsend",
                           mpi_int_send(buf, count, dtype, dest, 
                                        tag, comm, 0, req));
}


int
MPI_Isend(void *buf, int count, MPI_Datatype dtype, 
          int dest, int tag, MPI_Comm comm, 
          MPI_Request *req)
{
    return mpi_error_check("MPI_Isend",
                           mpi_int_send(buf, count, dtype, dest, 
                                        tag, comm, 0, req));
}


int
MPI_Issend(void *buf, int count, MPI_Datatype dtype,
           int dest, int tag, MPI_Comm comm, 
           MPI_Request *req)
{
    return mpi_error_check("MPI_Issend",
                           mpi_int_send(buf, count, dtype, dest, 
                                        tag, comm, 1, req));
}


int
MPI_Rsend(void *buf, int count, MPI_Datatype dtype,
          int dest, int tag, MPI_Comm comm)
{
    MPI_Request req;
    int ret;

    ret = mpi_int_send(buf, count, dtype, dest, 
                       tag, comm, 0, &req);
    if (MPI_SUCCESS != ret) return mpi_error_check("MPI_Rsend", ret);

    return mpi_error_check("MPI_Rsend", 
                           mpi_int_request_wait(&req, MPI_STATUS_IGNORE));
}


int
MPI_Send(void *buf, int count, MPI_Datatype dtype,
         int dest, int tag, MPI_Comm comm)
{
    MPI_Request req;
    int ret;

    ret = mpi_int_send(buf, count, dtype, dest, 
                       tag, comm, 0, &req);
    if (MPI_SUCCESS != ret) return mpi_error_check("MPI_Send", ret);

    return mpi_error_check("MPI_Send",
                           mpi_int_request_wait(&req, MPI_STATUS_IGNORE));
}


int
MPI_Ssend(void *buf, int count, MPI_Datatype dtype,
          int dest, int tag, MPI_Comm comm)
{
    MPI_Request req;
    int ret;

    ret = mpi_int_send(buf, count, dtype, dest, 
                           tag, comm, 1, &req);
    if (MPI_SUCCESS != ret) return mpi_error_check("MPI_Ssend", ret);

    return mpi_error_check("MPI_Ssend",
                           mpi_int_request_wait(&req, MPI_STATUS_IGNORE));
}
