// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/*
** $Id:$
** Not really an MPI program! We run a loop on each node and
** count the number of iterations. Speedup should be strictly
** linear.
**
** April 2005, Rolf Riesen, Sandia National Laboratories
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "mpi.h"

#define DURATION	(20.0)



int
main(int argc, char **argv)
{

register int i;
register double x;
double end_time;
double start_time;
int rank, nnodes;


    MPI_Init (&argc, &argv);
    MPI_Comm_size (MPI_COMM_WORLD, &nnodes);
    MPI_Comm_rank (MPI_COMM_WORLD, &rank);


    if (rank == 0)   {
        printf("\n\nFloatingpoint Performance Test\n");
        printf("===++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++"
            "=++=++=++=++===\n");
    }

    /* Run for twenty seconds */
    start_time= MPI_Wtime();
    end_time= start_time + DURATION;
    x= 1.0;
    i= 0;
    while (MPI_Wtime() < end_time)   {
	x= x * 1.000002 + 0.000003;
	i++;
    }
    
    MPI_Barrier(MPI_COMM_WORLD);
    if (rank == 0)   {
	    printf("Done\n");
	    printf("    x is %lf\n", x);
	    printf("    i is %d\n", i);
	    printf("    Gflops is %lf\n", (((i * 2) / 1000000000.0)/ DURATION) * nnodes);
    }

    MPI_Finalize();

    exit(0);
}

/* ===++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=++=== */
