// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/*
** $Id:$
** Test some MPI collectives.
** Copyright Rolf Riesen 2009
** 
*/
#include <stdio.h>
#include <unistd.h>	/* For sleep() */
#include <mpi.h>
#include <malloc/malloc.h>	// For malloc_printf()
#define printf malloc_printf


#define BUF_SIZE	(24)	/* number of ints */



int
main(int argc, char *argv[])
{

int my_rank, nproc;
int buf[BUF_SIZE];
int root;
int cnt, i;
int my_value, calc;


    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &nproc);
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    cnt= BUF_SIZE;
    root= 0;

    printf("*** %d entering MPI_Barrier()\n", my_rank);
    MPI_Barrier(MPI_COMM_WORLD);
    printf("*** %d back from MPI_Barrier()\n", my_rank);
    printf("*** %d ------------------------------------------------------------------------------------------\n", my_rank);



    printf("*** %d Initializing for MPI_Bcast()\n", my_rank);
    for (i= 0; i < BUF_SIZE; i++)   {
	if (my_rank == root)   {
	    buf[i]= i + 1000;
	} else   {
	    buf[i]= 0;
	}
    }

    printf("*** %d entering MPI_Bcast()\n", my_rank);
    MPI_Bcast(buf, cnt, MPI_INT, root, MPI_COMM_WORLD);
    printf("*** %d back from MPI_Bcast()\n", my_rank);

    for (i= 0; i < BUF_SIZE; i++)   {
	if (my_rank == root)   {
	    /* Nothing to do */
	} else   {
	    /* Everybody else checks */
	    if (buf[i] != i + 1000)   {
		fprintf(stderr, "Node %d: wrong value %d != %d at %d\n", my_rank, buf[i],
		    i + 1000, i);
		return -1;
	    }
	}
    }
    printf("*** %d received the MPI_Bcast() data correctly\n", my_rank);
    printf("*** %d ------------------------------------------------------------------------------------------\n", my_rank);



    printf("*** %d Initializing MPI_Reduce()\n", my_rank);
    /* Each node contributes its own node number */
    my_value= my_rank;
    buf[0]= 0;
    cnt= 1;

    printf("*** %d entering MPI_Reduce()\n", my_rank);
    MPI_Reduce(&my_value, &(buf[0]), cnt, MPI_INT, MPI_SUM, root, MPI_COMM_WORLD);
    printf("*** %d back from MPI_Reduce()\n", my_rank);

    if (my_rank == root)   {
	calc= 0;
	for (i= 0; i < nproc; i++)   {
	    calc= calc + i;
	}
	printf("*** %d Expected result is %d, got %d.\n", my_rank, calc, buf[0]);
    }
    printf("*** %d ------------------------------------------------------------------------------------------\n", my_rank);

    MPI_Finalize();
    return 0;

}  /* end of main() */
