// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/*
** $Id:$
** Send a token around a ring
** Copyright Rolf Riesen 2008
** 
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>	/* For sleep() */
#include <mpi.h>
#include <malloc/malloc.h>	// For malloc_printf()
#define printf malloc_printf



int
main(int argc, char *argv[])
{

int my_rank, nproc;
int i;
int token;
int tag;
int rounds;
int left, right;
MPI_Status status;


    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &nproc);
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    /* Initialize some variables */
    token= 0;
    tag= 0;
    rounds= 2;

    right= (my_rank + 1) % nproc;
    left= (my_rank - 1 + nproc) % nproc;


    /* Rank 0 starts the token */
    if (my_rank == 0)   {
	MPI_Send(&token, 1, MPI_INT, right, tag, MPI_COMM_WORLD);
    }

    for (i= 0; i < rounds; i++)   {
	MPI_Recv(&token, 1, MPI_INT, left, tag,
	    MPI_COMM_WORLD, &status);
	printf("Rank %d: received token %d from rank %d.\n",
	    my_rank, token, left);

	token= token + 1;
	// sleep(1);

	if ((my_rank != 0) || (i < (rounds - 1)))   {
	    /*
	    ** Only send it on if we are not rank 0,
	    ** or there are more rounds to go.
	    **/
	    MPI_Send(&token, 1, MPI_INT, right, tag,
		MPI_COMM_WORLD);
	}
    }

    MPI_Finalize();
    exit(0);

}  /* end of main() */
