// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/*
** $Id:$
** Rolf Riesen, September 2005, Sandia National Laboratories
**
** This is a simple MPI ping pong program we want to use to
** test the MPI node plugin.
*/

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <malloc/malloc.h>	// For malloc_printf()
#define printf malloc_printf



/*
** Some constants
*/
#define LEN	(10)



/*
** Let's get started
*/
int
main(int argc, char *argv[])
{

int my_rank, nnodes;
int last;
int i;
int rc;
void *data;
int sbuf[LEN];
int rbuf[LEN];
MPI_Status status;
double t, t2;


    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &nnodes);
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
    last= nnodes - 1;

    printf("Hello, I'm the pingpong program running on node %d/%d\n", my_rank, nnodes);


    if (my_rank == 0)   {
	/* Fill the send buffer with data */
	for (i= 0; i < LEN; i++)   {
	    sbuf[i]= i;
	}
	/* Clear the receive buffer */
	for (i= 0; i < LEN; i++)   {
	    rbuf[i]= -1;
	}
	data= sbuf;
	printf("    [%d] sending some data (%d bytes)\n", my_rank, LEN * sizeof(int));
	t= MPI_Wtime();
	rc= MPI_Send(data, LEN, MPI_INT, last, 0xAAAA, MPI_COMM_WORLD);
	if (rc != MPI_SUCCESS)   {
	    fprintf(stderr, "....[PINGPONG %d]______MPI_Send() failed\n", my_rank);
	    exit(-1);
	}

	data= rbuf;
	MPI_Recv(data, LEN, MPI_INT, last, 0xBBBB, MPI_COMM_WORLD, &status);
	printf("    [%d] the data has come back. Cheking it...\n", my_rank);
	for (i= 0; i < LEN; i++)   {
	    if (sbuf[i] != rbuf[i])   {
		fprintf(stderr, "........Data compare error at %d: %d != %d\n", i, sbuf[i], rbuf[i]);
		exit(-1);
	    }
	    rbuf[i]= -1;
	}

	t2= MPI_Wtime();
	printf("........Received data is correct. Time to execute %10.6fs\n", t2 - t);

    } else  if (my_rank == last)   {
	/* Clear the receive buffer */
	for (i= 0; i < LEN; i++)   {
	    rbuf[i]= -1;
	}
	data= rbuf;
	printf("    [%d] waiting for data.\n", my_rank);
	MPI_Recv(data, LEN, MPI_INT, 0, 0xAAAA, MPI_COMM_WORLD, &status);
	printf("    [%d] data has come in; sending it back.\n", my_rank);
	MPI_Send(data, LEN, MPI_INT, 0, 0xBBBB, MPI_COMM_WORLD);
    } else   {
	/* The other nodes do nothing */
    }

    MPI_Finalize();
    exit(0);

}  /* end of main() */
