// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


/*
** $Id:$
**
** Send a token around a ring. The owner of the token communicates with
** every other node, one by one, and then sends on the token to the next
** node in the ring. This tests all possible connections and should
** fail when links are inoperative.
**
** Copyright Rolf Riesen 2009
** 
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>	/* For sleep() */
#include <mpi.h>

#define TOKEN		(10)
#define P2PTEST		(20)
#define MAX_ROUNDS	(2)

int
main(int argc, char *argv[])
{

int my_rank, nproc;
int i;
int value;
int tag;
int steps;
int max_steps;
int left, right;
MPI_Status status;


    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &nproc);
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    /* Initialize some variables */
    value= TOKEN;
    tag= 0;
    steps= 0;

    max_steps= nproc * MAX_ROUNDS;

    right= (my_rank + 1) % nproc;
    left= (my_rank - 1 + nproc) % nproc;


    /* Rank 0 starts the token */
    if (my_rank == 0)   {
	printf("Doing %d rounds\n", max_steps / nproc);
	MPI_Send(&value, 1, MPI_INT, right, tag, MPI_COMM_WORLD);
    }

    while (steps <= max_steps)   {
	MPI_Recv(&value, 1, MPI_INT, MPI_ANY_SOURCE, tag,
	    MPI_COMM_WORLD, &status);

	if (value == TOKEN)   {
	    printf("Rank %3d: received token from rank %3d.\n", my_rank, left);

	    /* I have the token, communicate with all my neighbors */
	    printf("          Ping-pong to node ");
	    fflush(stdout);
	    for (i= 0; i < nproc; i++)   {
		if (i != my_rank)   {
		    value= P2PTEST;
		    printf("%d -- ", i);
		    fflush(stdout);
		    MPI_Send(&value, 1, MPI_INT, i, tag, MPI_COMM_WORLD);
		    MPI_Recv(&value, 1, MPI_INT, i, tag, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		    printf("%d, ", i);
		    fflush(stdout);
		}
	    }
	    printf("\n");

	    value= TOKEN;
	    MPI_Send(&value, 1, MPI_INT, right, tag, MPI_COMM_WORLD);

	} else   {
	    /* I received a test message. Send it back and keep waiting */
	    MPI_Send(&value, 1, MPI_INT, status.MPI_SOURCE, tag, MPI_COMM_WORLD);
	}
	steps++;
	if ((steps % nproc) == 0)   {
	    sleep(5);
	}
    }

    MPI_Finalize();
    exit(0);

}  /* end of main() */
