// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


#include <stdio.h>
#include <string.h>		// For strlen
#include <malloc/malloc.h>	// For malloc_printf()
#define malloc_printf	printf	/* Somewhat works now on x86 */
#include <netsim.h>


#define IDSTR	"test_send __________ "
#define MAX_MSG	(1024)


int 
main(int argc, char *argv[])
{

int rc;
int my_rank;
int nranks;
int dest;
char *data= "This is a test message";
int data_len;
int sync;
uint64_t match;
uint64_t user;
uint64_t ignore;
double t1, t2;
char rcv_buf[MAX_MSG];
struct netsim_cqe_t cqe;


    t1= netsim_get_clock();
    rc= netsim_init();
    my_rank= netsim_rank();
    nranks= netsim_size();
    dest= nranks - 1;

    if (nranks < 2)   {
	malloc_printf("Need at least two ranks for this test!\n");
	return 0;
    }

    if (my_rank == 0)   {
	/* I'm the sender */
	malloc_printf("%s My rank is %d/%d, sending to %d\n", IDSTR, my_rank, nranks, dest);
	data_len= strlen(data) + 1;
	sync= 0;
	match= 0x123456789abcdef0ll;
	user= 0xa5a5a5a5a5a5a5a5ll;
	netsim_tx_start(dest, data, data_len, sync, match, user);

	malloc_printf("%s Waiting for send completion....\n", IDSTR);

	while (netsim_cq_poll(&cqe) == -1)   {
	    // Wait for the send to complete
	}
	malloc_printf("%s Send of %d bytes complete\n", IDSTR, data_len);

    } else if (my_rank == dest)   {
	/* I'm the receiver */
	malloc_printf("%s My rank is %d/%d. I'll be receiving from %d\n", IDSTR, my_rank, nranks, 0);
	data_len= MAX_MSG;
	match= 0x123456789abcdef0ll;
	ignore= 0x0000000000000000ll;
	user= 0xbeef;
	netsim_rx_start(rcv_buf, data_len, match, ignore, user);

	malloc_printf("%s Waiting for recv completion....\n", IDSTR);

	while (netsim_cq_poll(&cqe) == -1)   {
	    // Wait for the send to complete
	}

	malloc_printf("%s Received %d bytes: \"%s\"\n", IDSTR, cqe.data_len, rcv_buf);

    } else   {
	/* I don't do anything */
	malloc_printf("%s My rank is %d/%d. Not participating\n", IDSTR, my_rank, nranks);
    }

    rc= netsim_finalize();
    t2= netsim_get_clock();
    malloc_printf("%s Time to execute: %gs\n", IDSTR, t2 - t1);

    return 0;

}  /* end of main() */
