// Copyright 2009-2010 Sandia Corporation. Under the terms
// of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
// Government retains certain rights in this software.
// 
// Copyright (c) 2009-2010, Sandia Corporation
// All rights reserved.
// 
// This file is part of the SST software package. For license
// information, see the LICENSE file in the top level directory of the
// distribution.


#include <stdio.h>
#include <malloc/malloc.h>	// For malloc_printf()
#include <netsim.h>
#include "netsim_internal.h"
#include "ppc_syscall.h"

#define IDSTR	"trap_test __________ "
#define malloc_printf	printf


#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)
#define NETSIM_SYS_ENTER_STR TOSTRING(NETSIM_SYS_ENTER)


netsim_params_t params;

void print_regs(char *str);


int 
main(int argc, char *argv[])
{

int rc= 99;
int routine= NETSIM_FINALIZE;
int params_length= sizeof(params);


    // print_regs("Before");
    asm volatile ( 
	"mr r3, %1\n\t"				// Which netsim function to call
	"mr r4, %2\n\t"				// Address of shared data struct
	"mr r5, %3\n\t"				// Size of shared data struct
	"li r0, "NETSIM_SYS_ENTER_STR"\n\t"	// Set to syscall
	"sc\n\t"				// make the sys call
	"mr %0, r3"				// collect results
	: "=r" (rc)				// Output operands
	: "r" (routine),			// Input operands
	  "r" (&params),
	  "r" (params_length)
        );

// If I list the clobbered registers, like that:
//	: "r0", "r3", "r4", "r5");		// List of clobbered registers (in addition to in/out)
// Then setting r[3] inside genericProc does not return it here.
// Don't know why.
// If I don't list them, then I can set r[5] inside genericProc
// and use it as the return code....

    // print_regs("After");
    malloc_printf("%s Back from netsim_finalize(), rc = %d\n", IDSTR, rc);
    malloc_printf("%s                         routine = %d\n", IDSTR, routine);
    malloc_printf("%s                   params_length = %d\n", IDSTR, params_length);
    return 0;

}  /* end of main() */


void
print_regs(char *str)
{

int regs[32];
int i, j;

    asm volatile ("mr %0, r0\n\t" : "=r" (regs[0]));
    asm volatile ("mr %0, r1\n\t" : "=r" (regs[1]));
    asm volatile ("mr %0, r2\n\t" : "=r" (regs[2]));
    asm volatile ("mr %0, r3\n\t" : "=r" (regs[3]));
    asm volatile ("mr %0, r4\n\t" : "=r" (regs[4]));
    asm volatile ("mr %0, r5\n\t" : "=r" (regs[5]));
    asm volatile ("mr %0, r6\n\t" : "=r" (regs[6]));
    asm volatile ("mr %0, r7\n\t" : "=r" (regs[7]));
    asm volatile ("mr %0, r8\n\t" : "=r" (regs[8]));
    asm volatile ("mr %0, r9\n\t" : "=r" (regs[9]));
    asm volatile ("mr %0, r10\n\t" : "=r" (regs[10]));
    asm volatile ("mr %0, r11\n\t" : "=r" (regs[11]));
    asm volatile ("mr %0, r12\n\t" : "=r" (regs[12]));
    asm volatile ("mr %0, r13\n\t" : "=r" (regs[13]));
    asm volatile ("mr %0, r14\n\t" : "=r" (regs[14]));
    asm volatile ("mr %0, r15\n\t" : "=r" (regs[15]));
    asm volatile ("mr %0, r16\n\t" : "=r" (regs[16]));
    asm volatile ("mr %0, r17\n\t" : "=r" (regs[17]));
    asm volatile ("mr %0, r18\n\t" : "=r" (regs[18]));
    asm volatile ("mr %0, r19\n\t" : "=r" (regs[19]));
    asm volatile ("mr %0, r20\n\t" : "=r" (regs[20]));
    asm volatile ("mr %0, r21\n\t" : "=r" (regs[21]));
    asm volatile ("mr %0, r22\n\t" : "=r" (regs[22]));
    asm volatile ("mr %0, r23\n\t" : "=r" (regs[23]));
    asm volatile ("mr %0, r24\n\t" : "=r" (regs[24]));
    asm volatile ("mr %0, r25\n\t" : "=r" (regs[25]));
    asm volatile ("mr %0, r26\n\t" : "=r" (regs[26]));
    asm volatile ("mr %0, r27\n\t" : "=r" (regs[27]));
    asm volatile ("mr %0, r28\n\t" : "=r" (regs[28]));
    asm volatile ("mr %0, r29\n\t" : "=r" (regs[29]));
    asm volatile ("mr %0, r30\n\t" : "=r" (regs[30]));
    asm volatile ("mr %0, r31\n\t" : "=r" (regs[31]));

    malloc_printf("---------------------------------------- %s\n", str);
    for (i= 0; i < 8; i++)   {
        for (j= 0; j < 4; j++)   {
            malloc_printf("r[%12d] %d   ", i * 4 + j, regs[i*4+j]);
        }
        malloc_printf("\n");
    }
    malloc_printf("======================================== %s\n", str);
}
